@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2001-2009 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2001-2018 HP Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@ import locale
 from .sixext.moves import html_entities, urllib2_request, urllib2_parse, urllib2_error
 from .sixext import PY3, to_unicode, to_bytes_utf8, to_string_utf8, BytesIO, StringIO, subprocess
 from . import os_utils
+import importlib
 try:
     import xml.parsers.expat as expat
     xml_expat_avail = True
@@ -61,22 +62,15 @@ except ImportError:
 
 try:
     import dbus
-    from dbus import SystemBus, lowlevel
+    from dbus import SystemBus, lowlevel, SessionBus
     dbus_avail=True
 except ImportError:
     dbus_avail=False
 
-try:
-    import hashlib # new in 2.5
+import hashlib
 
-    def get_checksum(s):
-        return hashlib.sha1(s).hexdigest()
-
-except ImportError:
-    import sha # deprecated in 2.6/3.0
-
-    def get_checksum(s):
-        return sha.new(s).hexdigest()
+def get_checksum(s):
+    return hashlib.sha256(s).hexdigest()
 
 
 
@@ -107,25 +101,61 @@ ERROR_DIGITAL_SIGN_BAD =3
 MAJ_VER = sys.version_info[0]
 MIN_VER = sys.version_info[1]
 
-
-
 EXPECT_WORD_LIST = [
     pexpect.EOF, # 0
     pexpect.TIMEOUT, # 1
-    "Continue?", # 2 (for zypper)
-    "passwor[dt]:", # en/de/it/ru
-    "kennwort", # de?
-    "password for", # en
-    "mot de passe", # fr
-    "contraseña", # es
-    "palavra passe", # pt
-    "口令", # zh
-    "wachtwoord", # nl
-    "heslo", # czech
-    "密码",
-    "Lösenord", #sv
+    u"Continue?", # 2 (for zypper)
+    u"passwor[dt]:", # en/de/it/ru
+    u"kennwort", # de?
+    u"password for", # en
+    u"mot de passe", # fr
+    u"contraseña", # es
+    u"palavra passe", # pt
+    u"口令", # zh
+    u"wachtwoord", # nl
+    u"heslo", # czech
+    u"密码",
+    u"Lösenord", #sv
 ]
 
+# Define the dictionary to map various distro names to standard names
+distro_name_dict = {
+    "unknown": "unknown",
+    "mepis": "mepis",
+    "debian": "debian",
+    "suse": "suse",
+    "opensuse": "suse",
+    "mandriva": "mandriva",
+    "fedora": "fedora",
+    "redhat": "rhel",
+    "rhel": "rhel",
+    "redhatenterprise" : "rhel",
+    "red hat enterprise linux": "rhel",
+    "slackware": "slackware",
+    "gentoo": "gentoo",
+    "redflag": "redflag",
+    "ubuntu": "ubuntu",
+    "xandros": "xandros",
+    "freebsd": "freebsd",
+    "linspire": "linspire",
+    "ark": "ark",
+    "pclinuxos": "pclinuxos",
+    "centos": "centos",
+    "igos": "igos",
+    "linuxmint": "linuxmint",
+    "mint": "linuxmint",
+    "linpus": "linpus",
+    "gos": "gos",
+    "boss": "boss",
+    "lfs": "lfs",
+    "manjarolinux": "manjarolinux",
+    "manjaro": "manjarolinux",
+    "zorin": "zorin",
+    "mxlinux": "mxlinux",
+    "mx": "mxlinux",
+    "elementary": "elementary",
+    "elementaryos": "elementary"
+}
 
 EXPECT_LIST = []
 for s in EXPECT_WORD_LIST:
@@ -149,7 +179,7 @@ def get_cups_systemgroup_list():
             return lis
 
     try:
-        lis = ((re.findall('SystemGroup [\w* ]*',fp.read()))[0].replace('SystemGroup ','')).split(' ')
+        lis = ((re.findall(r'SystemGroup [\w* ]*',fp.read()))[0].replace('SystemGroup ','')).split(' ')
     except IndexError:
         return lis
 
@@ -267,7 +297,8 @@ def walkFiles(root, recurse=True, abs_paths=False, return_folders=False, pattern
     try:
         names = os.listdir(root)
     except os.error:
-        raise StopIteration
+       #raise StopIteration
+        return
 
     pattern = pattern or '*'
     pat_list = pattern.split(';')
@@ -713,9 +744,15 @@ def canEnterGUIMode4(): # qt4
         log.warn("No display found.")
         return False
 
-    elif not checkPyQtImport4():
-        log.warn("Qt/PyQt 4 initialization failed.")
-        return False
+    # elif not checkPyQtImport4():
+    #     log.warn("Qt4/PyQt 4 initialization failed.")
+    #     return False
+    else:
+        try:
+            checkPyQtImport45()
+        except ImportError as e:
+            log.warn(e)
+            return False
 
     return True
 
@@ -776,10 +813,22 @@ def checkPyQtImport4():
         import PyQt4
         import ui4
     except ImportError:
-        log.error("HPLIP is not installed properly or is installed without graphical support. Please reinstall HPLIP")
-        return False
+        import PyQt5
+        import ui5
     else:
-        return True
+        log.debug("HPLIP is not installed properly or is installed without graphical support. Please reinstall HPLIP again")
+        return False
+    return True
+
+# def checkPyQtImport5():
+#     try:
+#         import PyQt5
+#         import ui5
+#     except ImportError:
+#         log.error("HPLIP is not installed properly or is installed without graphical support PyQt5. Please reinstall HPLIP")
+#         return False
+#     else:
+#         return True
 
 
 try:
@@ -937,7 +986,10 @@ else:
 
 
 def printable(s):
-    return s.translate(identity, unprintable)
+    if s:
+        return s.translate(identity, unprintable)
+    else:
+        return ""
 
 
 def any(S,f=lambda x:x):
@@ -1260,6 +1312,17 @@ def run(cmd, passwordObj = None, pswd_msg='', log_output=True, spinner=True, tim
     import io
     output = io.StringIO()
 
+    pwd_prompt_str = ""
+    if passwordObj and ('su' in cmd or 'sudo' in cmd) and os.geteuid() != 0:
+        pwd_prompt_str = passwordObj.getPasswordPromptString()
+        log.debug("cmd = %s pwd_prompt_str = [%s]"%(cmd, pwd_prompt_str))
+        if(pwd_prompt_str == ""):
+            passwd = passwordObj.getPassword(pswd_msg, 0)
+            pwd_prompt_str = passwordObj.getPasswordPromptString()
+            log.debug("pwd_prompt_str2 = [%s]"%(pwd_prompt_str))
+            if(passwd == ""):
+               return 127, ""
+
     try:
         child = pexpect.spawnu(cmd, timeout=timeout)
     except pexpect.ExceptionPexpect as e:
@@ -1277,15 +1340,22 @@ def run(cmd, passwordObj = None, pswd_msg='', log_output=True, spinner=True, tim
                 continue
 
             if child.before:
+                if(pwd_prompt_str and pwd_prompt_str not in EXPECT_LIST):
+                    log.debug("Adding %s to EXPECT LIST"%pwd_prompt_str)
+                    try:
+                        p = re.compile(pwd_prompt_str, re.I)
+                    except TypeError:
+                        EXPECT_LIST.append(pwd_prompt_str)
+                    else:
+                        EXPECT_LIST.append(p)
+                        EXPECT_LIST.append(pwd_prompt_str)
+
                 try:
                     output.write(child.before)
+                    if log_output:
+                        log.debug(child.before)
                 except Exception:
                     pass
-                if log_output:
-                    try:
-                        log.debug(child.before)
-                    except Exception:
-                        pass
 
             if i == 0: # EOF
                 break
@@ -1386,6 +1456,24 @@ def collapse_range(x): # x --> sorted list of ints
 
     return ''.join(s)
 
+def createBBSequencedFilename(basename, ext, dir=None, digits=3):
+    if dir is None:
+        dir = os.getcwd()
+
+    m = 0
+    for f in walkFiles(dir, recurse=False, abs_paths=False, return_folders=False, pattern='*', path=None):
+        r, e = os.path.splitext(f)
+
+        if r.startswith(basename) and ext == e:
+            try:
+                i = int(r[len(basename):])
+            except ValueError:
+                continue
+            else:
+                m = max(m, i)
+
+    return os.path.join(dir, "%s%0*d%s" % (basename, digits, m+1, ext))
+
 
 def createSequencedFilename(basename, ext, dir=None, digits=3):
     if dir is None:
@@ -1525,7 +1613,7 @@ def mixin(cls):
  # ------------------------- Usage Help
 USAGE_OPTIONS = ("[OPTIONS]", "", "heading", False)
 USAGE_LOGGING1 = ("Set the logging level:", "-l<level> or --logging=<level>", 'option', False)
-USAGE_LOGGING2 = ("", "<level>: none, info\*, error, warn, debug (\*default)", "option", False)
+USAGE_LOGGING2 = ("", r"<level>: none, info\*, error, warn, debug (\*default)", "option", False)
 USAGE_LOGGING3 = ("Run in debug mode:", "-g (same as option: -ldebug)", "option", False)
 USAGE_LOGGING_PLAIN = ("Output plain text only:", "-t", "option", False)
 USAGE_ARGS = ("[PRINTER|DEVICE-URI]", "", "heading", False)
@@ -1533,13 +1621,13 @@ USAGE_ARGS2 = ("[PRINTER]", "", "heading", False)
 USAGE_DEVICE = ("To specify a device-URI:", "-d<device-uri> or --device=<device-uri>", "option", False)
 USAGE_PRINTER = ("To specify a CUPS printer:", "-p<printer> or --printer=<printer>", "option", False)
 USAGE_BUS1 = ("Bus to probe (if device not specified):", "-b<bus> or --bus=<bus>", "option", False)
-USAGE_BUS2 = ("", "<bus>: cups\*, usb\*, net, bt, fw, par\* (\*defaults) (Note: bt and fw not supported in this release.)", 'option', False)
+USAGE_BUS2 = ("", r"<bus>: cups\*, usb\*, net, bt, fw, par\* (\*defaults) (Note: bt and fw not supported in this release.)", 'option', False)
 USAGE_HELP = ("This help information:", "-h or --help", "option", True)
 USAGE_SPACE = ("", "", "space", False)
 USAGE_EXAMPLES = ("Examples:", "", "heading", False)
 USAGE_NOTES = ("Notes:", "", "heading", False)
 USAGE_STD_NOTES1 = ("If device or printer is not specified, the local device bus is probed and the program enters interactive mode.", "", "note", False)
-USAGE_STD_NOTES2 = ("If -p\* is specified, the default CUPS printer will be used.", "", "note", False)
+USAGE_STD_NOTES2 = (r"If -p\* is specified, the default CUPS printer will be used.", "", "note", False)
 USAGE_SEEALSO = ("See Also:", "", "heading", False)
 USAGE_LANGUAGE = ("Set the language:", "--loc=<lang> or --lang=<lang>. Use --loc=? or --lang=? to see a list of available language codes.", "option", False)
 USAGE_LANGUAGE2 = ("Set the language:", "--lang=<lang>. Use --lang=? to see a list of available language codes.", "option", False)
@@ -1551,10 +1639,15 @@ USAGE_INTERACTIVE_MODE = ("Run in interactive mode:", "-i or --interactive", "op
 if sys_conf.get('configure', 'ui-toolkit', 'qt3') == 'qt3':
     USAGE_USE_QT3 = ("Use Qt3:",  "--qt3 (Default)",  "option",  False)
     USAGE_USE_QT4 = ("Use Qt4:",  "--qt4",  "option",  False)
-else:
+    USAGE_USE_QT5 = ("Use Qt5:",  "--qt5",  "option",  False)
+elif sys_conf.get('configure', 'ui-toolkit', 'qt4') == 'qt4':
     USAGE_USE_QT3 = ("Use Qt3:",  "--qt3",  "option",  False)
     USAGE_USE_QT4 = ("Use Qt4:",  "--qt4 (Default)",  "option",  False)
-
+    USAGE_USE_QT5 = ("Use Qt5:",  "--qt5",  "option",  False)
+elif sys_conf.get('configure', 'ui-toolkit', 'qt5') == 'qt5':
+    USAGE_USE_QT3 = ("Use Qt3:",  "--qt3",  "option",  False)
+    USAGE_USE_QT4 = ("Use Qt4:",  "--qt4",  "option",  False)
+    USAGE_USE_QT5 = ("Use Qt5:",  "--qt5 (Default)",  "option",  False)
 
 def ttysize(): # TODO: Move to base/tui
     ln1 = subprocess.getoutput('stty -a').splitlines()[0]
@@ -1762,7 +1855,7 @@ encoding: utf8
 
     elif typ == 'man':
         log.info('.TH "%s" 1 "%s" Linux "User Manuals"' % (crumb, version))
-        log.info(".SH NAME\n%s \- %s" % (crumb, title))
+        log.info(r".SH NAME\n%s \- %s" % (crumb, title))
 
         for line in text_list:
             text1, text2, format, trailing_space = line
@@ -1793,7 +1886,7 @@ encoding: utf8
                 log.info(text1)
 
         log.info(".SH AUTHOR")
-        log.info("HPLIP (Hewlett-Packard Linux Imaging and Printing) is an")
+        log.info("HPLIP (HP Linux Imaging and Printing) is an")
         log.info("HP developed solution for printing, scanning, and faxing with")
         log.info("HP inkjet and laser based printers in Linux.")
 
@@ -1805,7 +1898,7 @@ encoding: utf8
         log.info("contact the HPLIP Team.")
 
         log.info(".SH COPYRIGHT")
-        log.info("Copyright (c) 2001-15 Hewlett-Packard Development Company, L.P.")
+        log.info("Copyright (c) 2001-18 HP Development Company, L.P.")
         log.info(".LP")
         log.info("This software comes with ABSOLUTELY NO WARRANTY.")
         log.info("This is free software, and you are welcome to distribute it")
@@ -1824,7 +1917,7 @@ def log_title(program_name, version, show_ver=True): # TODO: Move to base/module
 
     log.info(log.bold("%s ver. %s" % (program_name, version)))
     log.info("")
-    log.info("Copyright (c) 2001-15 Hewlett-Packard Development Company, LP")
+    log.info("Copyright (c) 2001-18 HP Development Company, LP")
     log.info("This software comes with ABSOLUTELY NO WARRANTY.")
     log.info("This is free software, and you are welcome to distribute it")
     log.info("under certain conditions. See COPYING file for more details.")
@@ -1861,7 +1954,7 @@ def unescape(text):
             except KeyError:
                 pass
         return text # leave as is
-    return re.sub("&#?\w+;", fixup, text)
+    return re.sub(r"&#?\w+;", fixup, text)
 
 
 # Adds HTML or XML character references and entities from a text string
@@ -1914,7 +2007,7 @@ def Is_HPLIP_older_version(installed_version, available_version):
     log.debug("HPLIP Installed_version=%s  Available_version=%s"%(installed_version,available_version))
     cnt = 0
     Is_older = False
-    pat=re.compile('''(\d{1,})([a-z]{1,})''')
+    pat=re.compile(r'''(\d{1,})([a-z]{1,})''')
     try:
         while cnt <len(installed_array) and cnt <len(available_array):
 
@@ -1997,7 +2090,7 @@ def chunk_write(response, out_fd, chunk_size =8192, status_bar = downLoad_status
 def download_from_network(weburl, outputFile = None, useURLLIB=False):
     retValue = -1
 
-    if weburl is "" or weburl is None:
+    if weburl == "" or weburl == None:
         log.error("URL is empty")
         return retValue, ""
 
@@ -2018,7 +2111,7 @@ def download_from_network(weburl, outputFile = None, useURLLIB=False):
 
         if useURLLIB:
 		
-            sys.stdout.write("Download in progress..........")
+            #sys.stdout.write("Download in progress..........")
             try:
                 response = urllib2_request.urlopen(weburl)    
                 file_fd = open(outputFile, 'wb')
@@ -2065,7 +2158,7 @@ def sendEvent(event_code,device_uri, printer_name, username="", job_id=0, title=
 
     log.debug("send_message() entered")
     args = [device_uri, printer_name, event_code, username, job_id, title, pipe_name]
-    msg = lowlevel.SignalMessage('/', DBUS_SERVICE, 'Event')
+    msg = lowlevel.SignalMessage(path='/', interface=DBUS_SERVICE, name='Event')
     msg.append(signature='ssisiss', *args)
     SystemBus().send_message(msg)
     log.debug("send_message() returning")
@@ -2355,3 +2448,230 @@ def find_pip():
         return 'pip-python'
     else:
         log.error("python pip command not found. Please install '%s' package(s) manually"%depends_to_install_using_pip)
+
+
+def check_lan():
+    try:
+        x = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        x.connect(('1.2.3.4', 56))
+        x.close()
+        return True
+    except socket.error:
+        return False
+ 
+def extract_xml_chunk(data):
+    if data.find('\r\n\r\n'):
+        index = data.find('\r\n\r\n')
+        data = data[index+4:]
+    if data[0:1] != '<':            # Check for source encoding chunked or content length in http respose header.
+        size = -1
+        temp = ""
+        while size:
+            index = data.find('\r\n')
+            size = int(data[0:index+1], 16)
+            temp = temp + data[index+2:index+2+size]
+            data = data[index+2+size+2:len(data)]
+        data = temp
+    return data
+
+
+def checkPyQtImport45():
+    try:
+        import PyQt5
+        return "PyQt5"
+    except ImportError as e:
+        log.debug(e)
+
+    try:
+        import PyQt4
+        return "PyQt4"
+    except ImportError as e:
+        log.debug(e)
+
+    raise ImportError("GUI Modules PyQt4 and PyQt5 are not installed")
+
+
+def ui_status():
+    _ui_status = ""
+    try:
+        _ui_status = checkPyQtImport45()
+        log.note("Using GUI Module %s" % _ui_status)
+        return _ui_status
+    except ImportError as e:
+        log.error(e)
+
+
+def import_dialog(ui_toolkit):
+    if ui_toolkit == "qt4":
+        try:
+            from PyQt4.QtGui import QApplication
+            log.debug("Using PyQt4")
+            return  (QApplication, "ui4")
+        except ImportError as e:
+            log.error(e)
+            sys.exit(1)
+    elif ui_toolkit == "qt5":
+        try:
+            from PyQt5.QtWidgets import QApplication
+            log.debug("Using PyQt5")
+            return (QApplication, "ui5")
+        except ImportError as e:
+            log.error(e)
+            sys.exit(1)
+        else:
+            log.error("Unable to load Qt support. Is it installed?")
+            sys.exit(1)
+
+
+def dyn_import_mod(mod_name_as_str):
+    components = mod_name_as_str.split('.')
+    mod = __import__(mod_name_as_str)
+    for comp in components[1:]:
+        mod = getattr(mod, comp)
+    return mod
+
+def import_ext(ext_name):
+    try:
+        return importlib.import_module(ext_name)
+    except ImportError:
+        import sysconfig
+        sys.path.append(sysconfig.get_path('platlib'))
+        return importlib.import_module(ext_name)
+
+def get_distro_name(passwordObj = None):
+    log.debug("Determining distro...")
+    name, ver = '', '0.0'
+    found = False
+    distro_release_name = str()
+    # Getting distro information using platform module
+    try:
+        import platform
+        try:
+            name = platform.dist()[0].lower()
+            ver = platform.dist()[1]
+        except AttributeError:
+            import distro
+            name = distro.linux_distribution()[0].lower()
+            ver = distro.linux_distribution()[1]
+            distro_release_name = distro.distro_release_attr('name')
+        if not name:
+            found = False
+            log.debug("Not able to detect distro")
+        else:
+            found = True
+            log.debug("Able to detect distro")
+    except ImportError:
+        found = False
+        log.debug("Not able to detect distro using python")
+
+    # Getting distro information using lsb_release command
+    # platform return 'redhat' even for 'RHEL' or 'arch' for ManjaroLinux so re-reading using
+    # lsb_release.
+    if not found or name == 'redhat' or name == 'arch':
+        lsb_rel = which("lsb_release", True)
+        if lsb_rel:
+            log.debug("Using 'lsb_release -is/-rs' to determine distro")
+            status, name = run(lsb_rel + ' -is', passwordObj)
+            if not status and name:
+                status, ver = run(lsb_rel + ' -rs', passwordObj)
+                if not status and ver:
+                    ver = ver.lower().strip()
+                    found = True
+
+    # Getting distro information using /etc/issue file
+    if not found:
+        try:
+            name = open('/etc/issue', 'r').read().lower().strip()
+                
+        except IOError:
+            found = False
+        else:
+            found = True
+            for n in name.split():
+                m = n
+                if '.' in n:
+                    m = '.'.join(n.split('.')[:2])
+
+                try:
+                    ver = float(m)
+                except ValueError:
+                    try:
+                        ver = int(m)
+                    except ValueError:
+                        ver = '0.0'
+    if "welcome" in name:
+        found = False
+    #search in etc/os-release
+    if not found:
+        try:
+            os_release = open('/etc/os-release', 'r')
+            for line in os_release:
+                if line.lower().startswith('name='):
+                    name = line.split('"')[1]
+                if line.lower().startswith('version='):
+                    ver = line.split('"')[1]
+                found = True
+        except:
+            found = False          
+    # Updating the distro name and version.
+    if found:
+        name = name.lower().strip()
+        #convert the name to a standard name based on distro name dictionary
+        #for example "welcome to MX Linux" gets converted to "mxlinux"
+        # Check if any value from the distro_name_dict is a substring of name
+        for key, standard_name in distro_name_dict.items():
+            if key in name:
+                name = standard_name
+                break
+
+        log.debug("Distro name=%s" % name)
+
+
+        log.debug("Distro version=%s" % ver)
+        if name == "rhel" and ver[0] == "5" and ver[1] == ".":
+            ver = "5.0"
+        elif name == "rhel" and ver[0] == "6" and ver[1] == ".":
+            ver = "6.0"
+        if 'MX' in distro_release_name:
+            name = "mxlinux"
+            ver = distro_release_name[3:5]
+        if 'manjaro' in name.lower():
+            version = ver.split('.')
+            ver = version[0] +'.'+version[1]
+        
+    else:
+        log.warn("Failed to get the distro information.")
+        name, ver = 0, '0.0'
+
+    log.debug("distro=%s, distro_version=%s" %(str(name),str(ver)))
+    return (name,ver)
+                
+def readAuthType():
+    try:
+        import os
+        from grp import getgrnam
+        authType = 'su'
+        user = os.getenv('USER')
+        try:
+            members = getgrnam('wheel').gr_mem
+        except KeyError:
+            try:
+                members = getgrnam('sudo').gr_mem
+            except:
+                log.warn(" can not get user group " )
+                return authType     
+        if user in members:
+            authType = 'sudo'
+        else:
+            authType = 'su'
+    except :
+        log.warn("unable to determine auth_type ")
+    return authType
+
+def sanitize_filename(filename):
+    # Remove any path traversal sequences
+    filename = os.path.basename(filename)
+    # Ensure the filename only contains safe characters
+    if not re.match(r'^[\w\-. ]+$', filename):
+        raise ValueError("Invalid filename")
+    return filename
\ No newline at end of file
