@@ -1,7 +1,7 @@
 /*****************************************************************************\
   CommonDefinitions.h : common header
 
-  Copyright (c) 1996 - 2009, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2015, HP Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -12,7 +12,7 @@
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
-  3. Neither the name of Hewlett-Packard nor the names of its
+  3. Neither the name of HP nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.
 
@@ -48,7 +48,6 @@
 #include <syslog.h>
 #include <assert.h>
 #include <time.h>
-#include <sys/timeb.h>
 #include <string.h>
 
 #ifndef _GNU_SOURCE
@@ -441,7 +440,6 @@ typedef struct JobAttributes_s
 {
     int                media_source;
     int                color_mode;
-    int                HPSPDClass;
     DUPLEXMODE         e_duplex_mode;
     int                print_borderless;
     int                krgb_mode;
@@ -466,6 +464,8 @@ typedef struct JobAttributes_s
     int                integer_values[16];
     int                printer_platform_version;
     int                pre_process_raster;
+    int                HPSPDClass;
+    int 	           args_duplex_mode;
 } JobAttributes;
 
 #endif // COMMON_DEFINITIONS_H
